
/**
 *	S3C (Simple Server-side Cache)
 *	Server-side Object Caching Framework 
 *
 *	Copyright (C) 2005  Kamran Zafar
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	Contact Info:
 *	Email: xeus.man@gmail.com
 */

package xeus.s3c;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;

import xeus.s3c.event.CacheEvent;
import xeus.s3c.event.CacheListener;
import xeus.s3c.exception.SlotNotAvailableException;

/**
 * @author Kamran Zafar
 *  
 */
public class S3Cache implements Serializable, Cache {

    private HashMap cache;

    private List listeners;

    private String name;

    private int timeout;

    private int slots;

    private int slotCount;
    private Invalidator iThread;

    public S3Cache(String name, int timeout, int slots) {
        this(name);
        this.timeout = timeout;
        this.slots = slots;
    }

    public S3Cache(String name) {
        this.name = name;
        this.timeout = -1;
        this.slots = 10;
        this.slotCount = 0;
        cache = new HashMap();
        listeners = new ArrayList();
    }

    public void initialize() {
        if (timeout > 0)
        {
            iThread=new Invalidator(this);
            iThread.start();
        }
    }
    
    public void invalidate()
    {
        if(iThread!=null)
            iThread.interrupt();
        
        fireCacheInvalidatedEvent(new CacheEvent(this));
    }
    
    public void updated()
    {
        fireCachUpdatedEvent(new CacheEvent(this));
    }

    public Slot register(String slot) throws SlotNotAvailableException {
        if (slotCount == slots)
            throw new SlotNotAvailableException("No more slots available");

        if (cache.get(slot) == null) {
            Slot space = new Slot(slot);
            cache.put(slot, space);
            slotCount++;
            
            CacheEvent event=new CacheEvent(this,slot);
            fireSlotRegisteredEvent(event);
            return space;
        } else
            throw new SlotNotAvailableException(
                    "Slot with this name already registered");
    }
    
    public Slot register(String slot,int sTimeout) throws SlotNotAvailableException {
        if (slotCount == slots)
            throw new SlotNotAvailableException("No more slots available");

        if (cache.get(slot) == null) {
            Slot space = new Slot(slot);
            space.setTimeout(sTimeout);
            cache.put(slot, space);
            slotCount++;
            
            CacheEvent event=new CacheEvent(this,slot);
            fireSlotRegisteredEvent(event);
            return space;
        } else
            throw new SlotNotAvailableException(
                    "Slot with this name already registered");
    }

    public void unregister(String slot) throws SlotNotAvailableException {
        if (cache.get(slot) != null) {
            cache.remove(slot);
            slotCount--;
            
            CacheEvent event=new CacheEvent(this,slot);
            fireSlotUnregisteredEvent(event);
        } else
            throw new SlotNotAvailableException("Slot not found");
    }

    public void addCacheListener(CacheListener listener) {
        listeners.add(listener);
    }

    public void removeCacheListener(CacheListener listener) {
        listeners.remove(listener);
    }

    protected void fireSlotRegisteredEvent(CacheEvent event) {
        for (int i = 0; i < listeners.size(); i++)
            ((CacheListener) listeners.get(i)).slotRegistered(event);
    }
    
    protected void fireSlotUnregisteredEvent(CacheEvent event) {
        for (int i = 0; i < listeners.size(); i++)
            ((CacheListener) listeners.get(i)).slotUnregistered(event);
    }
    
    protected void fireCacheInvalidatedEvent(CacheEvent event) {
        for (int i = 0; i < listeners.size(); i++)
            ((CacheListener) listeners.get(i)).cacheInvalidated(event);
    }

    protected void fireCachUpdatedEvent(CacheEvent event) {
        for (int i = 0; i < listeners.size(); i++)
            ((CacheListener) listeners.get(i)).cacheUpdated(event);
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public int getSlots() {
        return slots;
    }

    public void setSlots(int slots) {
        this.slots = slots;
    }

    public Slot lookup(String slot) throws SlotNotAvailableException {
        if (cache.get(slot) != null)
            return (Slot) cache.get(slot);
        else
            throw new SlotNotAvailableException();
    }

    public Collection slots() {
        return cache.values();
    }

    public int getTimeout() {
        return timeout;
    }

    public void setTimeout(int timeout) {
        this.timeout = timeout;
    }

    private class Invalidator extends Thread implements Serializable {
        S3Cache cache;

        public Invalidator(S3Cache cache) {
            this.cache = cache;
        }

        public void run() {
            while (true) {
                try {
                    sleep(cache.getTimeout());
                } catch (InterruptedException e) {
                    continue;
                }
                cache.fireCacheInvalidatedEvent(new CacheEvent(cache));
            }
        }
    }
}